/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */


package cse466.packets;


public class AdjustGlobalsPacket extends CSE466Packet {
    public int node0;
    public int transmitPower;
    public int startledHopCount;
    public int lifeSpanMin;
    public int lifeSpanMax;
    public int minTempo;
    public int maxTempo;
    public int minBreedTime;
    public int maxBreedTime;
    public int ageTimerLength;
    public int inbreedingThreshold;
    public int inbreedingLimit;
    public int nightADCThreshold;
    public int mutationPercent;
    public int restScaleFactor;
    public int minVolume;
    public int maxVolume;

  public AdjustGlobalsPacket() {
    node0 = 0;
    transmitPower = 1;
    startledHopCount = 3;
    lifeSpanMin = 10;
    lifeSpanMax = 100;
    minTempo = 1;
    maxTempo = 3;
    minBreedTime = 100;
    maxBreedTime = 1000;
    ageTimerLength = 100;
    inbreedingThreshold = 50;
    inbreedingLimit = 5;
    nightADCThreshold = 10;
    mutationPercent = 5;
    restScaleFactor = 5;
    minVolume = 1;
    maxVolume = 8;

  }

  public int getProperAMType(){
    return 50;
  }

  public byte[] setFromBytes(byte[] fromArray){
    byte[] me = super.setFromBytes(fromArray);
    node0               = bytesToInt(me[ 0], me[ 1]);
    transmitPower       = bytesToInt(me[ 2], me[ 3]);
    startledHopCount    = bytesToInt(me[ 4], me[ 5]);
    lifeSpanMin         = bytesToInt(me[ 6], me[ 7]);
    lifeSpanMax         = bytesToInt(me[ 8], me[ 9]);
    minTempo            = bytesToInt(me[10], me[11]);
    maxTempo            = bytesToInt(me[12], me[13]);
    minBreedTime        = bytesToInt(me[14], me[15]);
    maxBreedTime        = bytesToInt(me[16], me[17]);
    ageTimerLength      = bytesToInt(me[18], me[19]);
    inbreedingThreshold = bytesToInt(me[20], me[21]);

    inbreedingLimit     = 0x00ff & (int)me[22];
    nightADCThreshold   = 0x00ff & (int)me[23];
    mutationPercent     = 0x00ff & (int)me[24];
    restScaleFactor     = 0x00ff & (int)me[25];
    minVolume           = 0x00ff & (int)me[26];
    maxVolume           = 0x00ff & (int)me[27];
    
    return subArray(me, 28);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 28];
    ret[ 0] = intToLSB(node0);
    ret[ 1] = intToMSB(node0);
    ret[ 2] = intToLSB(transmitPower);
    ret[ 3] = intToMSB(transmitPower);
    ret[ 4] = intToLSB(startledHopCount);
    ret[ 5] = intToMSB(startledHopCount);
    ret[ 6] = intToLSB(lifeSpanMin);
    ret[ 7] = intToMSB(lifeSpanMin);
    ret[ 8] = intToLSB(lifeSpanMax);
    ret[ 9] = intToMSB(lifeSpanMax);
    ret[10] = intToLSB(minTempo);
    ret[11] = intToMSB(minTempo);
    ret[12] = intToLSB(maxTempo);
    ret[13] = intToMSB(maxTempo);
    ret[14] = intToLSB(minBreedTime);
    ret[15] = intToMSB(minBreedTime);
    ret[16] = intToLSB(maxBreedTime);
    ret[17] = intToMSB(maxBreedTime);
    ret[18] = intToLSB(ageTimerLength);
    ret[19] = intToMSB(ageTimerLength);
    ret[20] = intToLSB(inbreedingThreshold);
    ret[21] = intToMSB(inbreedingThreshold);
    ret[22] = intToLSB(inbreedingLimit);
    ret[23] = intToLSB(nightADCThreshold);
    ret[24] = intToLSB(mutationPercent);
    ret[25] = intToLSB(restScaleFactor);
    ret[26] = intToLSB(minVolume);
    ret[27] = intToLSB(maxVolume);

    copyIntoArray(ret, childByteArray, 28);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("AG     node0               : " + Integer.toHexString(node0));
    System.out.println("AG     transmitPower       : " + Integer.toHexString(transmitPower));
    System.out.println("AG     startledHopCount    : " + Integer.toHexString(startledHopCount));
    System.out.println("AG     lifeSpanMin         : " + Integer.toHexString(lifeSpanMin));
    System.out.println("AG     lifeSpanMax         : " + Integer.toHexString(lifeSpanMax));
    System.out.println("AG     minTempo            : " + Integer.toHexString(minTempo));    
    System.out.println("AG     maxTempo            : " + Integer.toHexString(maxTempo));
    System.out.println("AG     minBreedTime        : " + Integer.toHexString(minBreedTime));
    System.out.println("AG     maxBreedTime        : " + Integer.toHexString(maxBreedTime));
    System.out.println("AG     ageTimerLength      : " + Integer.toHexString(ageTimerLength));
    System.out.println("AG     inbreedingThreshold : " + Integer.toHexString(inbreedingThreshold));
    System.out.println("AG     inbreedingLimit     : " + Integer.toHexString(inbreedingLimit));
    System.out.println("AG     nightADCThreshold   : " + Integer.toHexString(nightADCThreshold));
    System.out.println("AG     mutationPercent     : " + Integer.toHexString(mutationPercent));
    System.out.println("AG     restScaleFactor     : " + Integer.toHexString(restScaleFactor));
    System.out.println("AG     minVolume           : " + Integer.toHexString(minVolume));
    System.out.println("AG     maxVolume           : " + Integer.toHexString(maxVolume));
  }

  public boolean equals(AdjustGlobalsPacket other) {
    if(node0                != other.node0               ) return false;    
    if(transmitPower        != other.transmitPower       ) return false;    
    if(startledHopCount     != other.startledHopCount    ) return false;    
    if(lifeSpanMin          != other.lifeSpanMin         ) return false;    
    if(lifeSpanMax          != other.lifeSpanMax         ) return false;    
    if(minTempo             != other.minTempo            ) return false;    
    if(maxTempo             != other.maxTempo            ) return false;    
    if(minBreedTime         != other.minBreedTime        ) return false;    
    if(maxBreedTime         != other.maxBreedTime        ) return false;
    if(maxTempo             != other.maxTempo            ) return false;    
    if(ageTimerLength       != other.ageTimerLength      ) return false;    
    if(inbreedingThreshold  != other.inbreedingThreshold ) return false;    
    if(inbreedingLimit      != other.inbreedingLimit     ) return false;
    if(nightADCThreshold    != other.nightADCThreshold   ) return false;
    if(mutationPercent      != other.mutationPercent     ) return false;    
    if(restScaleFactor      != other.restScaleFactor     ) return false;    
    if(minVolume            != other.minVolume           ) return false;    
    if(maxVolume            != other.maxVolume           ) return false;
    return super.equals(other);
  }
}

