/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */


package cse466.packets;


public class AdjustGlobalsPacket extends CSE466Packet {
    public int node0;
	public int currentTime;
	public int repetition;
	public int minListen;
	public int maxListen;
	public int Threshold;
	public int minThreshold;
	public int probability;
	public int silence;
	public int transmitPower;
	public int startledHopCount;

  public AdjustGlobalsPacket() {
    node0 = 0;
	currentTime = 0;
	repetition = 3;
	minListen = 2000;
	maxListen = 15000;
	Threshold = 600;
	minThreshold = 100;
	probability = 10;
	silence = 10;
	transmitPower = 0x0f;
	startledHopCount = 1;
  }

  public int getProperAMType(){
    return 50;
  }

  public byte[] setFromBytes(byte[] fromArray){
    byte[] me = super.setFromBytes(fromArray);
    node0               = bytesToInt(me[ 0], me[ 1]);
    currentTime			= bytesToInt32(me[ 2], me[ 3], me[ 4], me[ 5]);
	repetition			= bytesToInt(me[ 6], me[ 7]);
	minListen			= bytesToInt(me[ 8], me[ 9]);
	maxListen			= bytesToInt(me[10], me[11]);
	Threshold			= bytesToInt(me[12], me[13]);
	minThreshold		= bytesToInt(me[14], me[15]);
	probability			= bytesToInt(me[16], me[17]);
	silence				= bytesToInt(me[18], me[19]);
	transmitPower		= bytesToInt(me[20], me[21]);
	startledHopCount	= bytesToInt(me[22], me[23]);

    return subArray(me, 24);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 24];
    ret[ 0] = intToLSB(node0);
    ret[ 1] = intToMSB(node0);
    ret[ 2] = intToLSB(currentTime);
    ret[ 3] = intToMSB(currentTime);
    ret[ 4] = intToLSBHigh(currentTime);
    ret[ 5] = intToMSBHigh(currentTime);
	ret[ 6] = intToLSB(repetition);
    ret[ 7] = intToMSB(repetition);
	ret[ 8] = intToLSB(minListen);
    ret[ 9] = intToMSB(minListen);
    ret[10] = intToLSB(maxListen);
    ret[11] = intToMSB(maxListen);
    ret[12] = intToLSB(Threshold);
    ret[13] = intToMSB(Threshold);
    ret[14] = intToLSB(minThreshold);
    ret[15] = intToMSB(minThreshold);
    ret[16] = intToLSB(probability);
    ret[17] = intToMSB(probability);
    ret[18] = intToLSB(silence);
    ret[19] = intToMSB(silence);
    ret[20] = intToLSB(transmitPower);
    ret[21] = intToMSB(transmitPower);
    ret[22] = intToLSB(startledHopCount);
    ret[23] = intToMSB(startledHopCount);

    copyIntoArray(ret, childByteArray, 24);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("AG     node0               : " + Integer.toHexString(node0));
    System.out.println("AG     currentTime         : " + Integer.toHexString(currentTime));
    System.out.println("AG     repetition          : " + Integer.toHexString(repetition));
    System.out.println("AG     minListen           : " + Integer.toHexString(minListen));
    System.out.println("AG     maxListen           : " + Integer.toHexString(maxListen));
    System.out.println("AG     Threshold           : " + Integer.toHexString(Threshold));    
    System.out.println("AG     minThreshold        : " + Integer.toHexString(minThreshold));
    System.out.println("AG     probability         : " + Integer.toHexString(probability));
    System.out.println("AG     silence             : " + Integer.toHexString(silence));
    System.out.println("AG     transmitPower       : " + Integer.toHexString(transmitPower));
    System.out.println("AG     startledHopCount    : " + Integer.toHexString(startledHopCount));
  }

  public boolean equals(AdjustGlobalsPacket other) {
    if(node0                != other.node0               ) return false;    
    if(currentTime          != other.currentTime         ) return false;    
    if(repetition           != other.repetition          ) return false;    
    if(minListen            != other.minListen           ) return false;    
    if(maxListen            != other.maxListen           ) return false;    
    if(Threshold            != other.Threshold           ) return false;    
    if(minThreshold         != other.minThreshold        ) return false;    
    if(probability          != other.probability         ) return false;    
    if(silence              != other.silence             ) return false;
    if(transmitPower        != other.transmitPower       ) return false;    
    if(startledHopCount     != other.startledHopCount    ) return false;
    return super.equals(other);
  }
}

