/*
 *
 * Copyright  2005, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */


package cse466.packets;

public class AlteredGenesPacket extends CSE466Packet {
  public int transmittingNodeNum;		//local # of startled node
  public int matings;								//# of matings since birth
	
  public int maxStrengthNode;				//Node num corresponding closest node
  public int maxStrength;						//RSSI value corresponding closest node
  public int minStrengthNode;				//Node num corresponding distant node
  public int minStrength;						//RSSI value corresponding most distant node
	
  public int mates_0;								//Local # of last 4 mates (genetic history)
  public int mates_1;								//Local # of last 4 mates (genetic history)
  public int mates_2;								//Local # of last 4 mates (genetic history)
  public int mates_3;								//Local # of last 4 mates (genetic history)
  public int matesCorrelation_0;			//Correlation value of last mates
  public int matesCorrelation_1;			//Correlation value of last mates
  public int matesCorrelation_2;			//Correlation value of last mates
  public int matesCorrelation_3;			//Correlation value of last mates
  public int numMutations;						//Number of mutations which occurred since 
																			//   last AlteredGenes pkt sent (8-bits

  public AlteredGenesPacket() {
    transmittingNodeNum = 0;
    matings = 3;
		
    maxStrengthNode = 2000;
    maxStrength = 1500;
    minStrengthNode = 600;
    minStrength = 100;
		
    mates_0 = 0;
    mates_1 = 0;
    mates_2 = 0;
    mates_3 = 0;
    matesCorrelation_0 = 0;
    matesCorrelation_1 = 0;
    matesCorrelation_2 = 0;
    matesCorrelation_3 = 0;
    numMutations = 1;
  }

  public int getProperAMType(){
    return 43;
  }

  public byte[] setFromBytes(byte[] fromArray){
    byte[] me = super.setFromBytes(fromArray);
    transmittingNodeNum = bytesToInt(me[ 0], me[ 1]);
    matings    		= bytesToInt(me[ 2], me[ 3]);
    maxStrengthNode     = bytesToInt(me[ 4], me[ 5]);
    maxStrength     	= bytesToInt(me[ 6], me[ 7]);
    minStrengthNode     = bytesToInt(me[ 8], me[ 9]);
    minStrength  	= bytesToInt(me[10], me[11]);
		
    mates_0             = bytesToInt(me[12], me[13]); // oldest mate in time
    mates_1             = bytesToInt(me[14], me[15]);
    mates_2             = bytesToInt(me[16], me[17]);
    mates_3             = bytesToInt(me[18], me[19]); // most recent mate
		
    matesCorrelation_0  = bytesToInt(me[20], me[21]); // oldest mate in time
    matesCorrelation_1  = bytesToInt(me[22], me[23]); 
    matesCorrelation_2  = bytesToInt(me[24], me[25]); 
    matesCorrelation_3  = bytesToInt(me[26], me[27]); // most recent mate
		
    numMutations        = 0x00ff & me[28];
    return subArray(me, 29);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 29];
    ret[ 0] = intToLSB(transmittingNodeNum);
    ret[ 1] = intToMSB(transmittingNodeNum);
    ret[ 2] = intToLSB(matings);
    ret[ 3] = intToMSB(matings);
    ret[ 4] = intToLSB(maxStrengthNode);
    ret[ 5] = intToMSB(maxStrengthNode);
    ret[ 6] = intToLSB(maxStrength);
    ret[ 7] = intToMSB(maxStrength);
    ret[ 8] = intToLSB(minStrengthNode);
    ret[ 9] = intToMSB(minStrengthNode);
    ret[10] = intToLSB(minStrength);
    ret[11] = intToMSB(minStrength);
		
    ret[12] = intToLSB(mates_0);
    ret[13] = intToMSB(mates_0);
    ret[14] = intToLSB(mates_1);
    ret[15] = intToMSB(mates_1);
    ret[16] = intToLSB(mates_2);
    ret[17] = intToMSB(mates_2);
    ret[18] = intToLSB(mates_3);
    ret[19] = intToMSB(mates_3);
    ret[20] = intToLSB(matesCorrelation_0);
    ret[21] = intToMSB(matesCorrelation_0);
    ret[22] = intToLSB(matesCorrelation_1);
    ret[23] = intToMSB(matesCorrelation_1);
    ret[24] = intToLSB(matesCorrelation_2);
    ret[25] = intToMSB(matesCorrelation_2);
    ret[26] = intToLSB(matesCorrelation_3);
    ret[27] = intToMSB(matesCorrelation_3);
		
    ret[28] = intToLSB(numMutations);
		
    copyIntoArray(ret, childByteArray, 29);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("ALG     transmittingNodeNum  : " + Integer.toHexString(transmittingNodeNum));
    System.out.println("ALG     matings              : " + Integer.toHexString(matings));
    System.out.println("ALG     maxStrengthNode      : " + Integer.toHexString(maxStrengthNode));
    System.out.println("ALG     maxStrength          : " + Integer.toHexString(maxStrength));
    System.out.println("ALG     minStrengthNode      : " + Integer.toHexString(minStrengthNode));
    System.out.println("ALG     minStrength          : " + Integer.toHexString(minStrength));
    System.out.println("ALG     mates_0              : " + Integer.toHexString(mates_0));
    System.out.println("ALG     mates_1              : " + Integer.toHexString(mates_1));
    System.out.println("ALG     mates_2              : " + Integer.toHexString(mates_2));
    System.out.println("ALG     mates_3              : " + Integer.toHexString(mates_3));
    System.out.println("ALG     matesCorrelation_0   : " + Integer.toHexString(matesCorrelation_0));
    System.out.println("ALG     matesCorrelation_1   : " + Integer.toHexString(matesCorrelation_1));
    System.out.println("ALG     matesCorrelation_2   : " + Integer.toHexString(matesCorrelation_2));
    System.out.println("ALG     matesCorrelation_3   : " + Integer.toHexString(matesCorrelation_3));
    System.out.println("ALG     numMutations         : " + Integer.toHexString(numMutations));
  }

  public boolean equals(AlteredGenesPacket other) {
    if(transmittingNodeNum       != other.transmittingNodeNum       ) return false;    
    if(matings                   != other.matings                   ) return false;    
    if(maxStrengthNode           != other.maxStrengthNode           ) return false;    
    if(maxStrength               != other.maxStrength               ) return false;    
    if(minStrengthNode           != other.minStrengthNode           ) return false;    
    if(minStrength               != other.minStrength               ) return false;    
    if(mates_0                   != other.mates_0                   ) return false;      
    if(mates_1                   != other.mates_1                   ) return false;      
    if(mates_2                   != other.mates_2                   ) return false;      
    if(mates_3                   != other.mates_3                   ) return false;    
    if(matesCorrelation_0        != other.matesCorrelation_0        ) return false;  
    if(matesCorrelation_1        != other.matesCorrelation_1        ) return false;  
    if(matesCorrelation_2        != other.matesCorrelation_2        ) return false; 
    if(matesCorrelation_3        != other.matesCorrelation_3        ) return false;  
    if(numMutations              != other.numMutations              ) return false;
    return super.equals(other);
  }
}

