/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

package cse466.packets;


public class CommandPacket extends CSE466Packet {

  public int node0;
  public int currentTime;
  public int startAddr;
  public int endAddr;
  public int LED;
  public int song;
  public int playtime;

  public CommandPacket() {
      node0 = 1;
	  currentTime = 0;
      startAddr = 0;
      endAddr = 0xffff;
	  LED = 0;
	  song = 0;
	  playtime = 0;
  }

  public int getProperAMType(){
    return 52;
  }


  public byte[] setFromBytes(byte[] fromArray){
    fromArray = super.setFromBytes(fromArray);
    node0                = bytesToInt(fromArray[ 0], fromArray[ 1]);
	currentTime          = bytesToInt32(fromArray[ 2], fromArray[ 3], fromArray [ 4], fromArray[ 5]);
    startAddr            = bytesToInt(fromArray[ 6], fromArray[ 7]);
    endAddr              = bytesToInt(fromArray[ 8], fromArray[ 9]);
    LED                  = bytesToInt(fromArray[10], fromArray[11]);
    song                 = bytesToInt(fromArray[12], fromArray[13]);
    playtime             = bytesToInt32(fromArray[14], fromArray[15], fromArray[16], fromArray[17]);
    return subArray(fromArray,  18);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 18];
    ret[ 0] = intToLSB(node0);
    ret[ 1] = intToMSB(node0);
    ret[ 2] = intToLSB(currentTime);
    ret[ 3] = intToMSB(currentTime);
    ret[ 4] = intToLSBHigh(currentTime);
    ret[ 5] = intToMSBHigh(currentTime);
	ret[ 6] = intToLSB(startAddr);
	ret[ 7] = intToMSB(startAddr);
	ret[ 8] = intToLSB(endAddr);
	ret[ 9] = intToMSB(endAddr);
    ret[10] = intToLSB(LED);
    ret[11] = intToMSB(LED);
    ret[12] = intToLSB(song);
    ret[13] = intToMSB(song);
    ret[14] = intToLSB(playtime);
    ret[15] = intToMSB(playtime);
    ret[16] = intToLSBHigh(playtime);
    ret[17] = intToMSBHigh(playtime);
	copyIntoArray(ret, childByteArray,  18);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("ST     node0               : " + Integer.toHexString(node0));
    System.out.println("ST     currentTime         : " + Integer.toHexString(currentTime));
    System.out.println("ST     startAddr           : " + Integer.toHexString(startAddr));
    System.out.println("ST     endAddr             : " + Integer.toHexString(endAddr));
    System.out.println("ST     LED                 : " + Integer.toHexString(LED));
    System.out.println("ST     song                : " + Integer.toHexString(song));
    System.out.println("ST     playtime            : " + Integer.toHexString(playtime));
  }

  public boolean equals(CommandPacket other){
    if(node0               != other.node0              ) return false;
    if(currentTime         != other.currentTime        ) return false;
    if(startAddr           != other.startAddr          ) return false;
    if(endAddr             != other.endAddr            ) return false;
    if(LED                 != other.LED                ) return false;
    if(song                != other.song               ) return false;
    if(playtime            != other.playtime           ) return false;
    return super.equals(other);
  }
}

