/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

package cse466.packets;


public class LedsPacket extends CSE466Packet {
  public int value;
  public int sourceAddr;

  public LedsPacket(){
    super();
    value = 0;
    sourceAddr = 0;
  }

  public int getProperAMType(){
    return 4;
  }

  public byte[] setFromBytes(byte[] fromArray) {
    fromArray = super.setFromBytes(fromArray);
    value = byteToInt(fromArray[0]);
    sourceAddr = bytesToInt(fromArray[1], fromArray[2]);
    return subArray(fromArray, 3);
  }
  
  protected byte[] getByteArray(byte[] childArray){
    byte[] ret = new byte[childArray.length + 3];
    ret[0] = intToByte(value);
    ret[1] = intToLSB(sourceAddr);
    ret[2] = intToMSB(sourceAddr);
    copyIntoArray(ret, childArray, 3);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("Leds   value               : " + Integer.toHexString(value));
    System.out.println("Leds   sourceAddr          : " + Integer.toHexString(sourceAddr));
  }

  public boolean equals(LedsPacket other){
    if(value      != other.value     ) return false;
    if(sourceAddr != other.sourceAddr) return false;
    return super.equals(other);
  }

}
