/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

package cse466.packets;

/*
uint16_t TransmittingNodeNum local # of node singing
uint16_t SequenceNum start at 1, increment each time you send this packet
uint16_t[8] phraseDNA[] List of phrases that birdie just sang where phraseDNA[0] is the first phrase sang in the last sequence.
uint16_t Tempo
uint16_t volume
*/

public class SangPhrasesPacket extends CSE466Packet {
  public int transmittingNodeNum;
  public int sequenceNum;
  public int phraseDNA_0;
  public int phraseDNA_1;
  public int phraseDNA_2;
  public int phraseDNA_3;
  public int phraseDNA_4;
  public int phraseDNA_5;
  public int phraseDNA_6;
  public int phraseDNA_7;
  public int tempo;
  public int volume;

  public SangPhrasesPacket() {
    transmittingNodeNum = 4;
    sequenceNum = 5;
    phraseDNA_0 = 0;
    phraseDNA_1 = 0;
    phraseDNA_2 = 0;
    phraseDNA_3 = 0;
    phraseDNA_4 = 0;
    phraseDNA_5 = 0;
    phraseDNA_6 = 0;
    phraseDNA_7 = 0;
    tempo = 7;
    volume = 8;
  }

  public int getProperAMType(){
    return 42;
  }


  public byte[] setFromBytes(byte[] fromArray){
    fromArray = super.setFromBytes(fromArray);
    transmittingNodeNum   = bytesToInt(fromArray[ 0], fromArray[ 1]);
    sequenceNum           = bytesToInt(fromArray[ 2], fromArray[ 3]);
    phraseDNA_0           = bytesToInt(fromArray[ 4], fromArray[ 5]);
    phraseDNA_1           = bytesToInt(fromArray[ 6], fromArray[ 7]);
    phraseDNA_2           = bytesToInt(fromArray[ 8], fromArray[ 9]);
    phraseDNA_3           = bytesToInt(fromArray[10], fromArray[11]);
    phraseDNA_4           = bytesToInt(fromArray[12], fromArray[13]);
    phraseDNA_5           = bytesToInt(fromArray[14], fromArray[15]);
    phraseDNA_6           = bytesToInt(fromArray[16], fromArray[17]);
    phraseDNA_7           = bytesToInt(fromArray[18], fromArray[19]);
    tempo                 = bytesToInt(fromArray[20], fromArray[21]);
    volume                = bytesToInt(fromArray[22], fromArray[23]);
    return subArray(fromArray, 24);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 24];
    ret[ 0] = intToLSB(transmittingNodeNum);
    ret[ 1] = intToMSB(transmittingNodeNum);
    ret[ 2] = intToLSB(sequenceNum);
    ret[ 3] = intToMSB(sequenceNum);
    ret[ 4] = intToLSB(phraseDNA_0);
    ret[ 5] = intToMSB(phraseDNA_0);
    ret[ 6] = intToLSB(phraseDNA_1);
    ret[ 7] = intToMSB(phraseDNA_1);
    ret[ 8] = intToLSB(phraseDNA_2);
    ret[ 9] = intToMSB(phraseDNA_2);
    ret[10] = intToLSB(phraseDNA_3);
    ret[11] = intToMSB(phraseDNA_3);
    ret[12] = intToLSB(phraseDNA_4);
    ret[13] = intToMSB(phraseDNA_4);
    ret[14] = intToLSB(phraseDNA_5);
    ret[15] = intToMSB(phraseDNA_5);
    ret[16] = intToLSB(phraseDNA_6);
    ret[17] = intToMSB(phraseDNA_6);
    ret[18] = intToLSB(phraseDNA_7);
    ret[19] = intToMSB(phraseDNA_7);
    ret[20] = intToLSB(tempo);
    ret[21] = intToMSB(tempo);
    ret[22] = intToLSB(volume);
    ret[23] = intToMSB(volume);
    copyIntoArray(ret, childByteArray, 24);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("SP     transmittingNodeNum : " + Integer.toHexString(transmittingNodeNum));
    System.out.println("SP     sequenceNum         : " + Integer.toHexString(sequenceNum));
    System.out.println("SP     phraseDNA_0         : " + Integer.toHexString(phraseDNA_0));
    System.out.println("SP     phraseDNA_1         : " + Integer.toHexString(phraseDNA_1));
    System.out.println("SP     phraseDNA_2         : " + Integer.toHexString(phraseDNA_2));
    System.out.println("SP     phraseDNA_3         : " + Integer.toHexString(phraseDNA_3));
    System.out.println("SP     phraseDNA_4         : " + Integer.toHexString(phraseDNA_4));
    System.out.println("SP     phraseDNA_5         : " + Integer.toHexString(phraseDNA_5));
    System.out.println("SP     phraseDNA_6         : " + Integer.toHexString(phraseDNA_6));
    System.out.println("SP     phraseDNA_7         : " + Integer.toHexString(phraseDNA_7));
    System.out.println("SP     tempo               : " + Integer.toHexString(tempo));
    System.out.println("SP     volume              : " + Integer.toHexString(volume));
  }

  public boolean equals(SangPhrasesPacket other){
    if(transmittingNodeNum != other.transmittingNodeNum) return false;
    if(sequenceNum         != other.sequenceNum        ) return false;
    if(phraseDNA_0         != other.phraseDNA_0        ) return false;
    if(phraseDNA_1         != other.phraseDNA_1        ) return false;
    if(phraseDNA_2         != other.phraseDNA_2        ) return false;
    if(phraseDNA_3         != other.phraseDNA_3        ) return false;
    if(phraseDNA_4         != other.phraseDNA_4        ) return false;
    if(phraseDNA_5         != other.phraseDNA_5        ) return false;
    if(phraseDNA_6         != other.phraseDNA_6        ) return false;
    if(phraseDNA_7         != other.phraseDNA_7        ) return false;
    if(tempo               != other.tempo              ) return false;
    if(volume              != other.volume             ) return false;
    return super.equals(other);
  }
}

