/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

package cse466.packets;

/*
uint16_t TransmittingNodeNum local # of node singing
uint16_t SequenceNum start at 1, increment each time you send this packet
uint16_t[8] phraseDNA[] List of phrases that birdie just sang where phraseDNA[0] is the first phrase sang in the last sequence.
uint16_t Tempo
uint16_t volume
*/

public class SangSongPacket extends CSE466Packet {
  public int transmittingNodeNum;
  public int currentTime;
  public int sequenceNum;
  public int songNum;
  public int songPointValue;
  public int maxPointSongNum;
  public int maxPointValue;
  public int runnerUpSongNum;
  public int runnerUpValue;
  public int minPointSongNum;
  public int minPointValue;

  public SangSongPacket() {
    transmittingNodeNum = 0;
	currentTime = 0;
    sequenceNum = 1;
    songNum = 0;
    songPointValue = 0;
	maxPointSongNum = 0;
    maxPointValue = 0;
    runnerUpSongNum = 0;
    runnerUpValue = 0;
    minPointSongNum = 0;
    minPointValue = 0;
  }

  public int getProperAMType(){
    return 42;
  }


  public byte[] setFromBytes(byte[] fromArray){
    fromArray = super.setFromBytes(fromArray);
    transmittingNodeNum   = bytesToInt(fromArray[ 0], fromArray[ 1]);
    currentTime           = bytesToInt32(fromArray[ 2], fromArray[ 3], fromArray[ 4], fromArray[ 5]);
    sequenceNum           = bytesToInt(fromArray[ 6], fromArray[ 7]);
    songNum               = bytesToInt(fromArray[ 8], fromArray[ 9]);
    songPointValue        = bytesToInt(fromArray[10], fromArray[11]);
    maxPointSongNum       = bytesToInt(fromArray[12], fromArray[13]);
    maxPointValue         = bytesToInt(fromArray[14], fromArray[15]);
    runnerUpSongNum       = bytesToInt(fromArray[16], fromArray[17]);
    runnerUpValue         = bytesToInt(fromArray[18], fromArray[19]);
    minPointSongNum       = bytesToInt(fromArray[20], fromArray[21]);
    minPointValue         = bytesToInt(fromArray[22], fromArray[23]);
    return subArray(fromArray, 24);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 24];
    ret[ 0] = intToLSB(transmittingNodeNum);
    ret[ 1] = intToMSB(transmittingNodeNum);
    ret[ 2] = intToLSB(currentTime);
    ret[ 3] = intToMSB(currentTime);
    ret[ 4] = intToLSBHigh(currentTime);
    ret[ 5] = intToMSBHigh(currentTime);
    ret[ 6] = intToLSB(sequenceNum);
    ret[ 7] = intToMSB(sequenceNum);
    ret[ 8] = intToLSB(songNum);
    ret[ 9] = intToMSB(songNum);
    ret[10] = intToLSB(songPointValue);
    ret[11] = intToMSB(songPointValue);
    ret[12] = intToLSB(maxPointSongNum);
    ret[13] = intToMSB(maxPointSongNum);
    ret[14] = intToLSB(maxPointValue);
    ret[15] = intToMSB(maxPointValue);
    ret[16] = intToLSB(runnerUpSongNum);
    ret[17] = intToMSB(runnerUpSongNum);
    ret[18] = intToLSB(runnerUpValue);
    ret[19] = intToMSB(runnerUpValue);
    ret[20] = intToLSB(minPointSongNum);
    ret[21] = intToMSB(minPointSongNum);
    ret[22] = intToLSB(minPointValue);
    ret[23] = intToMSB(minPointValue);
    copyIntoArray(ret, childByteArray, 24);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("SP     transmittingNodeNum : " + Integer.toHexString(transmittingNodeNum));
    System.out.println("SP     currentTime         : " + Integer.toHexString(currentTime));
    System.out.println("SP     sequenceNum         : " + Integer.toHexString(sequenceNum));
    System.out.println("SP     songNum             : " + Integer.toHexString(songNum));
    System.out.println("SP     songPointValue      : " + Integer.toHexString(songPointValue));
    System.out.println("SP     maxPointSongNum     : " + Integer.toHexString(maxPointSongNum));
    System.out.println("SP     maxPointValue       : " + Integer.toHexString(maxPointValue));
    System.out.println("SP     runnerUpSongNum     : " + Integer.toHexString(runnerUpSongNum));
    System.out.println("SP     runnerUpValue       : " + Integer.toHexString(runnerUpValue));
    System.out.println("SP     minPointSongNum     : " + Integer.toHexString(minPointSongNum));
    System.out.println("SP     minPointValue       : " + Integer.toHexString(minPointValue));
  }

  public boolean equals(SangSongPacket other){
    if(transmittingNodeNum != other.transmittingNodeNum) return false;
    if(currentTime         != other.currentTime        ) return false;
    if(sequenceNum         != other.sequenceNum        ) return false;
    if(songNum             != other.songNum            ) return false;
    if(songPointValue      != other.songPointValue     ) return false;
    if(maxPointSongNum     != other.maxPointSongNum    ) return false;
    if(maxPointValue       != other.maxPointValue      ) return false;
    if(runnerUpSongNum     != other.runnerUpSongNum    ) return false;
    if(runnerUpValue       != other.runnerUpValue      ) return false;
    if(minPointSongNum     != other.minPointSongNum    ) return false;
    if(minPointValue       != other.minPointValue      ) return false;
    return super.equals(other);
  }
}

