/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

package cse466.packets;

/*
uint16_t TransmittingNodeNum     local # of startled node
uint16_t HopCount                Number of hops remaining. If startled by radio message. 
                                 Decrement value before sending.
uint16_t StartledSeqNum          A randomly generated number that is stored to check to 
                                 see if you have been startled by this startle packet before. 
                                 Set by the bird startled by the light sensor.
*/

public class StartledPacket extends CSE466Packet {

  public int transmittingNodeNum;
  public int currentTime;
  public int hopCount;
  public int startledSeqNum;

  public StartledPacket() {
      transmittingNodeNum = 1;
	  currentTime = 0;
      hopCount = 1;
      startledSeqNum = 10;
  }

  public int getProperAMType(){
    return 60;
  }


  public byte[] setFromBytes(byte[] fromArray){
    fromArray = super.setFromBytes(fromArray);
    transmittingNodeNum   = bytesToInt(fromArray[ 0], fromArray[ 1]);
	currentTime           = bytesToInt32(fromArray[ 2], fromArray[ 3], fromArray [ 4], fromArray[ 5]);
    hopCount              = bytesToInt(fromArray[ 6], fromArray[ 7]);
    startledSeqNum        = bytesToInt(fromArray[ 8], fromArray[ 9]);
    return subArray(fromArray,  10);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 10];
    ret[ 0] = intToLSB(transmittingNodeNum);
    ret[ 1] = intToMSB(transmittingNodeNum);
    ret[ 2] = intToLSB(currentTime);
    ret[ 3] = intToMSB(currentTime);
    ret[ 4] = intToLSBHigh(currentTime);
    ret[ 5] = intToMSBHigh(currentTime);
	ret[ 6] = intToLSB(hopCount);
	ret[ 7] = intToMSB(hopCount);
	ret[ 8] = intToLSB(startledSeqNum);
	ret[ 9] = intToMSB(startledSeqNum);
    copyIntoArray(ret, childByteArray,  10);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("ST     transmittingNodeNum : " + Integer.toHexString(transmittingNodeNum));
    System.out.println("ST     currentTime         : " + Integer.toHexString(currentTime));
    System.out.println("ST     hopCount            : " + Integer.toHexString(hopCount));
    System.out.println("ST     startledSeqNum      : " + Integer.toHexString(startledSeqNum));
  }

  public boolean equals(StartledPacket other){
    if(transmittingNodeNum != other.transmittingNodeNum) return false;
    if(currentTime         != other.currentTime        ) return false;
    if(hopCount            != other.hopCount           ) return false;
    if(startledSeqNum      != other.startledSeqNum     ) return false;
    return super.equals(other);
  }
}

