/*
 *
 * Copyright  2005, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

package cse466.packets;

public class StopAndWaitPacket extends CSE466Packet {
  public int node0;
  public int startAddr;
  public int endAddr;
  
  public StopAndWaitPacket(){
    node0 = 0;
	startAddr = 0;
	endAddr = 0xffff;
  }

  public int getProperAMType(){
    return 51;
  }

  public byte[] setFromBytes(byte[] fromArray){
    byte[] me = super.setFromBytes(fromArray);
    node0 = bytesToInt(me[0], me[1]);
	startAddr = bytesToInt(me[2], me[3]);
	endAddr = bytesToInt(me[4], me[5]);
    return subArray(fromArray, 6);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 6];
    ret[0] = intToLSB(node0);
    ret[1] = intToMSB(node0);
    ret[2] = intToLSB(startAddr);
    ret[3] = intToMSB(startAddr);
    ret[4] = intToLSB(endAddr);
    ret[5] = intToMSB(endAddr);	  
    copyIntoArray(ret, childByteArray, 6);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("SAL    node0               : " + Integer.toHexString(node0));
	System.out.println("SAL    startAddr           : " + Integer.toHexString(startAddr));
	System.out.println("SAL    endAddr             : " + Integer.toHexString(endAddr));
  }
  
  public boolean equals(StopAndWaitPacket other){
    if(node0 != other.node0) return false;
	if(startAddr != other.startAddr) return false;
	if(endAddr != other.endAddr) return false;
    return super.equals(other);
  }
}
